const {ccclass, property} = cc._decorator;

@ccclass
export default class Helloworld extends cc.Component {

    video: HTMLVideoElement = null;
    gl: WebGLRenderingContext = null;
    texture: cc.Texture2D = null;

    onLoad() {
        this.video = document.createElement('video');
        this.gl = cc.renderer["device"]._gl;
        this.texture = new cc.Texture2D();
        this.texture.initWithData(new Uint8Array([0, 0, 0]), cc.Texture2D.PixelFormat.RGB888, 1, 1)
        this.node.addComponent(cc.Sprite).spriteFrame = new cc.SpriteFrame(this.texture);
    }
    
    update(dt) {
        this.accessWebCamera(this.video);
        this.updateTexture(this.gl, this.texture["_texture"], this.video);
    }
    
    updateTexture(gl, texture, video) {
        gl.bindTexture(gl.TEXTURE_2D, texture._glID);
        gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, video);
    }
    
    accessWebCamera(video) {
        return new Promise((resolve, reject) => {
            const mediaConstraints = { audio: false, video: {
                    width: this.node.width,
                    height: this.node.height,
                    brightness: {ideal: 2}
                }
            };

            navigator.mediaDevices.getUserMedia(mediaConstraints).then(mediaStream => {
                video.srcObject = mediaStream;
                video.setAttribute('play_video', true);
                video.onloadedmetadata = (e) => {
                    video.play();
                    resolve(video);
                }
            }).catch(err => {
                reject(err);
            });
        });
    }
}
